<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:as="http://www.oasis-open.org/committees/cam"
    xmlns:look="http://jcam.org.uk/LookupLists"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
    xmlns:camed="http://jcam.org.uk/editor">
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, January, 2009 -->
    <!--
        Purpose  : Parse CAM cxf file and create dictionary.
                   Inputs: Namespace of definitions to add to dictionary.
                              Existing dictionary to merge with.
        Original : 01/01/2009
        Initials : drrw
        Description : Parse exchange template and create domain dictionary library in XML lookup format for use in
                      building cross reference spreadsheet. Also creates CCTS style reference documentation.
       
        Updated  - Inits - Change
        01/01/09   drrw   Initial release 
        01/11/09   drrw   Merge mask detail if mask in dictionary empty
        01/16/09   drrw   Add UID logic 
        01/16/09   drrw   Correct merge action
        01/17/09   drrw   Minor fix to UID for namespace
        01/18/09   drrw   Default mergeMode to true, and display mergeMode value in header
        01/18/09   drrw   Resolve unique items in dictionary output
        01/19/09   drrw   Add context details
        01/19/09   drrw   Add where from source name details and parameter
        02/16/09   drrw   Handling of namespaces - children with no namespace should not remove items from dictionary
        03/25/09   drrw   Enhance dictionary layout to reflect CCTS semantics
        03/26/09   drrw   Add column for cardinality in BBIE/ABIE
        03/26/09   drrw   Add outputting of base CCTS component dictionary
        03/29/09   mmer removed CCTS default and printed out value passed in
        03/31/09   drrw   Add logic for "all" * asterix for targetNS selection parameter
        03/31/09   drrw   Add error check for input dictionary file same as output file
        03/31/09   drrw   Correct merge mode handling of CCTS
        05/23/09   drrw   Fix CCTSitem3 naming issue in merge details
        05/29/09   drrw   Remove namespace prefix from item name 
        06/06/09   drrw   Add new column for group from model ( annotations substitutionGroup )
        11/11/09   drrw   Missing dictionary entries for same item - different context
        11/12/09   drrw   New line separator &#xA; for Excel in context
        02/01/10   drrw   Add new columns to dictionary for children, label and notes
        02/05/10   drrw   Clean up children logic 
        02/05/10   drrw   Revise merge logic to exclude children, label, and notes
        02/17/10   drrw   Break language code list into separate attribute
        02/25/10   drrw   Note - when item has multiple mask options - currently output 1st one only
        02/25/10   drrw   Future enhancement to handle rule selection of type masks
        02/25/10   drrw   Generation of all context list of attributes - not just first one only
        02/27/10   drrw   Attribute generation for namespaced items and type and mask
        02/27/10   drrw   Note: use renamer to insert namespace into template names and paths
        03/16/10   drrw   Store code length limits in mask
        03/16/10   drrw   Distinguish number and decimal
        03/23/10   drrw   Selective filter on string length with string type
        08/05/10   drrw   Update dictionary content with latest annotations
        08/10/10   drrw   Add storing of children of root element if not AllRootElements
        08/10/10   drrw   Process attributeGroup annotation type
        08/12/10   drrw   Change root element processing - create children details - but put CCTS type of ROOT instead of ABIE
        01/17/11   drrw   Add support for lookup enumeration rules in mask
        06/15/11   drrw   Improve merge item selection logic
        06/15/11   drrw   Add sort of existing items during merge
        06/15/11   drrw   Prevent duplicate definitions from merge
        06/16/11   drrw   Dedup context
        06/16/11   drrw   Retain elements from original when not present in current template input (better merge)
        06/17/11   drrw   Dedup refinement for same name but different case / and merging context
        03/13/12   drrw   Add logic to retain namespace declarations on root element
        09/17/12   drrw   Fix for setLength() with one parameter; no hyphen
        12/10/12   drrw   Fix file handling for Ubuntu and file:// prefix
        12/12/12   drrw   Fix empty merge file handling
        ??/20/13   drrw   Fix conditional handling for dates format rules
        ??/20/13   drrw   Include setChoice rules handling into dictionary
        03/21/13   drrw   Action exclude() rules when generating dictionary components
        ??/20/13   drrw   Add column for context details; populate for namespaces and root elements
        ??/20/13   drrw   Support DBMapping annotations - but not specific column/table - just generic mapping level note
    -->
    <xsl:param name="DictionaryFile">NIEM-dictionary.xml</xsl:param>
    <xsl:param name="resultNamePath">new-dictionary.xml</xsl:param>
    <xsl:param name="targetNS"></xsl:param>
    <xsl:param name="mergeMode">true</xsl:param>
    <xsl:param name="whereFrom"></xsl:param>
    <xsl:param name="CCTS">true</xsl:param>
    <xsl:param name="insertNS"/>
    <xsl:param name="filterRules">true</xsl:param>

    <xsl:output name="resultXML" method="xml" indent="yes" omit-xml-declaration="yes" />
    
    <xsl:variable name="resultCCTS">
        <xsl:choose>
            <xsl:when test="contains($resultNamePath,'.')"><xsl:value-of select="concat(substring-before($resultNamePath,'.'),'-CC','.',substring-after($resultNamePath,'.'))"></xsl:value-of></xsl:when>
            <xsl:otherwise><xsl:value-of select="concat($resultNamePath,'-CC.xml')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="version">1.45</xsl:variable>
    <xsl:variable name="nlchar" select="'&#xA;'"/>
    <xsl:variable name="langCode">en-US</xsl:variable>
    <xsl:variable name="emptyDictionary"><ITEMSlookup xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:look="http://jcam.org.uk/LookupLists"/></xsl:variable>
    <xsl:variable name="DictionaryFilePath">
        <xsl:choose>
            <xsl:when test="contains($DictionaryFile,'://')"><xsl:value-of select="normalize-space(translate($DictionaryFile,'\','/'))"/></xsl:when>
            <xsl:when test="starts-with($DictionaryFile,'/')">file://<xsl:value-of select="translate($DictionaryFile,'\','/')"/></xsl:when>
            <xsl:when test="starts-with($DictionaryFile,'./')"><xsl:value-of select="translate($DictionaryFile,'\','/')"/></xsl:when>
            <xsl:when test="starts-with($DictionaryFile,'../')"><xsl:value-of select="translate($DictionaryFile,'\','/')"/></xsl:when>
            <xsl:when test="contains($DictionaryFile,'\')"><xsl:value-of select="if(starts-with($DictionaryFile,'file:')) then translate($DictionaryFile,'\','/') else concat('file:///',translate($DictionaryFile,'\','/'))"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$DictionaryFile"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="namespaces" select="/*/namespace::*"/>
    <xsl:variable name="debug">false</xsl:variable>
    <xsl:variable name="LookupProperties" select="if(string-length($DictionaryFilePath) &gt; 0 and(doc-available($DictionaryFilePath))) then document($DictionaryFilePath) else $emptyDictionary"/>
    <xsl:variable name="annotations">true</xsl:variable>
    <xsl:variable name="rootElement" select="as:CAM/as:AssemblyStructure/as:Structure/as:Element/@name[1]"/>
    <xsl:variable name="rootElementId" select="as:CAM/as:AssemblyStructure/as:Structure/as:Element[@name=$rootElement]/as:Attribute[@name='Id']/@exampleValue"/>
    
    <xsl:variable name="fromSource">
        <xsl:choose>
            <xsl:when test="string-length($whereFrom) &gt; 0 and($whereFrom &gt; ' ')"><xsl:value-of select="$whereFrom"/></xsl:when>
            <xsl:when test="string-length($rootElementId[1]) &gt; 0"><xsl:value-of select="concat($rootElement,'-',$rootElementId[1])"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$rootElement"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="rules" select="/"/>
    <xsl:variable name="ignoreDomainsNDR" select="'true'"/>
    
    <xsl:variable name="lookupTables" select="/as:CAM/as:Extension/*"/>
    
    <xsl:include href="word-utils.xsl"/>
    
    <xsl:include href="cam-NDR-functions.xsl"/>
    
    <xsl:template match="/">
        <xsl:if test="$DictionaryFilePath=replace($resultNamePath,'%20',' ')"><xsl:message terminate="yes">Error: Dictionary filename same as output results filename!</xsl:message></xsl:if>
        <xsl:message>
            Dictionary generator, version <xsl:value-of select="$version"/>
            Using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
            Writing output to - <xsl:value-of select="$resultNamePath"/>
            Merging mode    - <xsl:value-of select="$mergeMode"/> 
            For namespace - <xsl:value-of select="$targetNS"/>
            Source template - <xsl:value-of select="$fromSource"/>
            CCTS - <xsl:value-of select="$CCTS"/>
            Debug - <xsl:value-of select="$debug"/>
            Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>

        </xsl:message>
        <xsl:result-document exclude-result-prefixes="as camed" format="resultXML" indent="yes" href="{$resultNamePath}">
          
          <xsl:element name="ITEMSlookup">
              <xsl:copy-of select="$namespaces"/>
            
            <xsl:comment>
                Dictionary generator, version <xsl:value-of select="$version"/>
                Using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
                Writing output to - <xsl:value-of select="$resultNamePath"/>
                Merging mode    - <xsl:value-of select="$mergeMode"/> 
                For namespace - <xsl:value-of select="$targetNS"/>
                Source template - <xsl:value-of select="$fromSource"/>
		        CCTS - <xsl:value-of select="$CCTS"/>
                Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                <xsl:value-of select="'&#xD; &#xD;' "/>
            </xsl:comment>
            
            <xsl:message>
                Building dictionary started
            </xsl:message>                
            <xsl:message>
                - processing <xsl:value-of select="count(as:CAM/as:AssemblyStructure/as:Structure//as:Element)"/> elements
            </xsl:message>
                
            <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/child::*">
                <xsl:with-param name="mode">CAM</xsl:with-param>
            </xsl:apply-templates>
              
            <xsl:if test="$mergeMode='true'">
                <xsl:comment>Merged items from: <xsl:value-of select="$DictionaryFilePath"/><xsl:value-of select="'&#xD; &#xD;' "/></xsl:comment>
                <xsl:message>
                    Merging items from: <xsl:value-of select="$DictionaryFilePath"/>
                </xsl:message>
                <xsl:variable name="elements" select="as:CAM/as:AssemblyStructure/as:Structure//as:Element"/>
                <xsl:variable name="eachItem">
                    <xsl:apply-templates mode="list" select="$elements"/>
                    <xsl:apply-templates mode="list" select="$elements//as:Element"/>
                </xsl:variable>
                <!-- xsl:message>[<xsl:copy-of select="$eachItem"></xsl:copy-of>]</xsl:message -->
                <xsl:call-template name="mergeDetails">
                    <xsl:with-param name="elementItems" select="$eachItem"/>
                    <xsl:with-param name="mode">CAM</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
            
          </xsl:element>
          
        </xsl:result-document>
        
        <xsl:if test="$CCTS='true'">
            <xsl:message>
                Now generating CCTS dictionary
            </xsl:message>
            <xsl:result-document exclude-result-prefixes="as camed" format="resultXML" indent="yes" href="{$resultCCTS}">
            
            <CCTScatalogue>               
                <xsl:comment>
                    CCTS Dictionary generator, version <xsl:value-of select="$version"/>
                    Using dictionary - <xsl:value-of select="$DictionaryFilePath"/>
                    Writing output to - <xsl:value-of select="$resultNamePath"/>
                    Merging mode    - <xsl:value-of select="$mergeMode"/> 
                    For namespace - <xsl:value-of select="$targetNS"/>
                    Source template - <xsl:value-of select="$fromSource"/>
                    Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                    <xsl:value-of select="'&#xD; &#xD;' "/>
                </xsl:comment>
                
                <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/child::*">
                    <xsl:with-param name="mode">CCTS</xsl:with-param>
                </xsl:apply-templates>
                
                <xsl:if test="$mergeMode='true'">
                    <xsl:comment>Merged items from: <xsl:value-of select="$DictionaryFilePath"/><xsl:value-of select="'&#xD; &#xD;' "/></xsl:comment>
                    <xsl:message>
                        Merging CCTS items from: <xsl:value-of select="$DictionaryFilePath"/>
                    </xsl:message>
                    <xsl:variable name="elements" select="as:CAM/as:AssemblyStructure/as:Structure//as:Element"/>
                    <xsl:variable name="eachItem">
                        <xsl:apply-templates mode="list" select="$elements"/>
                        <xsl:apply-templates mode="list" select="$elements//as:Element"/>
                    </xsl:variable>
                    
                    <xsl:call-template name="mergeDetails">
                        <xsl:with-param name="elementItems" select="$eachItem"/>
                        <xsl:with-param name="mode">CCTS</xsl:with-param>
                     </xsl:call-template>
                </xsl:if>
            </CCTScatalogue>
            
        </xsl:result-document>
        </xsl:if>
        <xsl:message>
            Dictionary generation complete.
        </xsl:message>
    </xsl:template>
    
    <xsl:template match="as:Element" mode="list">
        <xsl:element name="item">
            <xsl:attribute name="name" select="@name"/>
            <xsl:attribute name="context" select="@path"/>
        </xsl:element>
        <xsl:apply-templates mode="list" select="as:Attribute"/>
        <xsl:apply-templates mode="list" select="as:Element"/>
    </xsl:template>
    
    <xsl:template match="as:Attribute" mode="list">
        <xsl:element name="item">
            <xsl:attribute name="name" select="@name"/>
            <xsl:attribute name="context" select="@path"/>
        </xsl:element>
        <xsl:apply-templates mode="list" select="as:Attribute"/>
    </xsl:template>
    
    <xsl:template match="as:Element">
        <xsl:param name="parentPath"/>
        <xsl:param name="mode"/>
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="itemCount" select="count(//as:Element[@name=$this])"/>
        <xsl:variable name="itemPath" select="//as:Element[@name=$this][1]/@path"/>
        <xsl:variable name="items" select="$LookupProperties//Property[@item = $itemName][1]"/><!-- removed upper-case() -->
        
        <xsl:variable name="fullContext" select="concat($parentPath,'/',@name)"/>
        <xsl:variable name="pathList" select="reverse(tokenize($fullContext,'/'))"/>
        <xsl:variable name="thisContext">
            <xsl:choose>
                <xsl:when test="count($pathList) &lt; 4">/<xsl:value-of select="$pathList[1]"/></xsl:when>
                <xsl:when test="count($pathList) &lt; 5">/<xsl:value-of select="$pathList[2]"/>/<xsl:value-of select="$pathList[1]"/></xsl:when>
                <xsl:otherwise><xsl:if test="$pathList[3] &gt; ' '">/<xsl:value-of select="$pathList[3]"/></xsl:if>/<xsl:value-of select="$pathList[2]"/>/<xsl:value-of select="$pathList[1]"/></xsl:otherwise>
            </xsl:choose> 
        </xsl:variable>
        
        <xsl:variable name="CCTS-type">
            <xsl:choose>
                <xsl:when test="@name=$rootElement">ROOT</xsl:when>
                <xsl:when test="count(node()/../child::as:Element/*) &gt; 0">ABIE</xsl:when>
                <xsl:otherwise>BBIE</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        
        <!-- Tree traversal path trace diagnostics -->
        <!-- xsl:variable name="depth" select="tokenize($thisContext,'/')"/><xsl:variable name="pad"><xsl:for-each select="$depth">_</xsl:for-each></xsl:variable>
       <xsl:message> <xsl:value-of select="concat($pad,$this)"/> [<xsl:value-of select="$thisContext"/>]</xsl:message>
        <xsl:message> <xsl:value-of select="concat($pad,'   ')"/> [<xsl:value-of select="$itemCount"/>][<xsl:value-of select="@path"/>][<xsl:value-of select="$itemPath"/>]</xsl:message -->
        
        <xsl:variable name="itemContextValues">
            <xsl:for-each select="//as:Element[@name=$this]">
                <xsl:choose>
                    <xsl:when test="position()=1"/><!-- ignore first item -->
                    <xsl:otherwise>
                        <xsl:variable name="chkPathList" select="reverse(tokenize(@path,'/'))"/>
                        <xsl:variable name="thisContextHere">
                            <xsl:choose>
                                <xsl:when test="count($chkPathList) &lt; 4">/<xsl:value-of select="$chkPathList[1]"/></xsl:when>
                                <xsl:when test="count($chkPathList) &lt; 5">/<xsl:value-of select="$chkPathList[2]"/>/<xsl:value-of select="$chkPathList[1]"/></xsl:when>
                                <xsl:otherwise><xsl:if test="$chkPathList[3] &gt; ' '">/<xsl:value-of select="$chkPathList[3]"/></xsl:if>/<xsl:value-of select="$chkPathList[2]"/>/<xsl:value-of select="$chkPathList[1]"/></xsl:otherwise>
                            </xsl:choose> 
                        </xsl:variable>
                        <context name="{@name}" context="{$thisContextHere}" seqn="{position()}"/>
                        <!-- xsl:message><xsl:value-of select="@name"/> - <xsl:value-of select="$thisContextHere"/> - <xsl:value-of select="position()"/></xsl:message -->
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:variable>
        
        <!-- xsl:message>show context: <xsl:copy-of select="$itemContextValues"/></xsl:message -->
        
        <xsl:choose>
            <xsl:when test="as:Rule/as:constraint[starts-with(@action,'exclude')]"/>
            <xsl:when test="$targetNS='*' or(($targetNS='' and(not(contains(@name,':')))) or(starts-with(@name,concat($targetNS,':'))))">  
                <!-- Output the first occurance only -->
                <xsl:if test="$itemCount=1 or($itemPath[1]=@path)">
                    <xsl:variable name="anno">
                        <!-- Loop getting annotations that are not "Definition"; store details with line breaks. -->
                        <xsl:for-each select="camed:annotation/camed:documentation">
                            <xsl:choose>
                                <xsl:when test="@type='Definition'"/><!-- ignore definition text -->
                                <xsl:when test="@type='Value'"/><!-- ignore value hints -->
                                <xsl:otherwise><xsl:value-of select="@type"/> = <xsl:value-of select="."/><xsl:value-of select="$nlchar"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="definition">
                        <xsl:choose>
                            <xsl:when test="camed:annotation/camed:documentation[@type='Definition']"><xsl:value-of select="camed:annotation/camed:documentation[@type='Definition']/text()"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="replace(as:Rule/as:constraint/as:annotation/as:documentation[@type='Definition']/text(),'\|','; ')"/></xsl:otherwise>
                        </xsl:choose> 
                    </xsl:variable>
                    <xsl:variable name="modelGroup">
                        <xsl:variable name="Grp1">
                        <xsl:choose>
                            <xsl:when test="camed:annotation/camed:documentation[@type='substitutionGroup']"><xsl:value-of select="camed:annotation/camed:documentation[@type='substitutionGroup']/text()"/></xsl:when>
                            <xsl:when test="parent::as:Element/camed:annotation/camed:documentation[@type='substitutionGroup']"><xsl:value-of select="parent::as:Element/camed:annotation/camed:documentation[@type='substitutionGroup']/text()"/></xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose> 
                        </xsl:variable>
                        <xsl:variable name="Grp2">
                        <xsl:choose>
                            <!-- Add attributeGroup also -->
                            <xsl:when test="camed:annotation/camed:documentation[@type='attributeGroup']"><xsl:value-of select="camed:annotation/camed:documentation[@type='attributeGroup']/text()"/></xsl:when>
                            <xsl:when test="parent::as:Element/camed:annotation/camed:documentation[@type='attributeGroup']"><xsl:value-of select="parent::as:Element/camed:annotation/camed:documentation[@type='attributeGroup']/text()"/></xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                        </xsl:variable>
                        <xsl:choose>
                            <xsl:when test="$Grp2=''"><xsl:value-of select="$Grp1"/></xsl:when>
                            <xsl:when test="$Grp1=''"><xsl:value-of select="$Grp2"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="concat($Grp1,$nlchar,$Grp2)"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="thisUID">
                        <xsl:choose>
                            <xsl:when test="camed:annotation/camed:documentation[@type='UID']"><xsl:value-of select="camed:annotation/camed:documentation[@type='UID']/text()"/></xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose> 
                    </xsl:variable>
                    <xsl:variable name="thisMask">
                        <xsl:choose>
                            <xsl:when test="$items//@type and(string-length($items//@mask) &gt; 0)"><xsl:value-of select="$items//@mask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'lookup(')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setDateMask')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setNumberMask')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setStringMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                <xsl:variable name="chkName" select="upper-case(@name)"/>
                                <xsl:variable name="findMaskParm"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:variable>
                                <xsl:variable name="maxLength" select="if(contains($findMaskParm,'-')) then xsd:integer(substring-after($findMaskParm,'-')) else xsd:integer($findMaskParm)"/>
                                <xsl:choose>
                                    <xsl:when test="$filterRules = 'false'"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:when>
                                    <xsl:when test="ends-with($chkName,'CODE') and($maxLength &lt; 41)"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:when>
                                    <xsl:when test="ends-with($chkName,'ID')"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:when>
                                    <xsl:when test="ends-with($chkName,'INDICATOR')"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:when>
                                    <xsl:when test="ends-with($chkName,'NUMBER') and($maxLength &lt; 36)"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:when>
                                    <xsl:when test="ends-with($chkName,'NUMBERTEXT') and($maxLength &lt; 36)"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:when>
                                    <xsl:when test="ends-with($chkName,'NAME')"/>
                                    <xsl:when test="ends-with($chkName,'TEXT')"/>
                                    <xsl:when test="ends-with($chkName,'COMMENT')"/>
                                    <xsl:when test="ends-with($chkName,'DESCRIPTION')"/>
                                    <xsl:when test="$maxLength &lt; 21"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$items//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/></xsl:call-template></xsl:when>
                                    <xsl:otherwise/>
                                </xsl:choose> 
                            </xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="thisType">
                        <xsl:choose>
                            <xsl:when test="$items//@type"><xsl:value-of select="$items//@type"/>@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'datatype')]"><xsl:value-of select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'datatype')][1]/@action,'('),')')"/>@</xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setType"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="replace(replace($thisMask,' \|',','),'''','')"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'lookup')]">list@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setDate"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setNumber"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:when test="$CCTS-type='ABIE'">@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:otherwise>string@<xsl:value-of select="$thisMask"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="UIDvalue">
                        <xsl:choose>
                            <xsl:when test="string-length($thisUID) &gt; 0"><xsl:value-of select="$thisUID"/></xsl:when>
                            <xsl:when test="$items//@UID"><xsl:value-of select="$items//@UID"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="if(contains(@name,':')) then concat(substring-before(@name,':'),'-') else 'cam-'"/><xsl:value-of select="generate-id()"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    
                    <xsl:variable name="cardinality">
                        <xsl:choose>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'makeRepeatable')]">repeatable<xsl:if test="as:Rule/as:constraint[contains(@action,'makeOptional')]">, optional</xsl:if></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'makeOptional')]">optional</xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                    </xsl:variable>
                    
                    <xsl:variable name="childContext">
                        <xsl:variable name="contextPath" select="@path"/>
                        <xsl:value-of select="$contextPath"/><xsl:value-of select="$nlchar"/>
                        <xsl:if test="not(@name='AllRootLevelTypeDefinitions')">
                            <xsl:for-each select="as:Element">
                                <xsl:value-of select="@name"/><xsl:value-of select="$nlchar"/>
                            </xsl:for-each>
                        </xsl:if>
                        <xsl:for-each select="as:Attribute">
                            <xsl:value-of select="concat('@',@name)"/><xsl:value-of select="$nlchar"/>
                        </xsl:for-each>
                    </xsl:variable>
                    
                    <xsl:variable name="children">
                        <xsl:choose>
                            <xsl:when test="count(tokenize($childContext,$nlchar)) &lt; 3"/><!-- No child elements or attributes -->
                            <xsl:when test="count($items) &gt; 0"><xsl:value-of select="if(contains($items//@children[1],$childContext)) then $items//@children[1] else if(string-length($items//@children[1]) &gt; 1) then concat($items//@children[1],'&#xA; ',$childContext) else $childContext"></xsl:value-of></xsl:when>
                            <xsl:when test="$annotations='true'"><xsl:value-of select="$childContext"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="$childContext"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>

                    <xsl:variable name="chkName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                    <xsl:variable name="firstLetter" select="substring($chkName,1,1)"/>
                   
                    <xsl:variable name="newname">
                                <xsl:choose>
                                    <xsl:when test="camed:WordMatch($chkName,'NIEM')='true'"><xsl:value-of select="$chkName"/></xsl:when>
                                    <xsl:when test="$firstLetter=$chkName and(upper-case($firstLetter)=$firstLetter)"/><!-- skip one char upper case name -->
                                    <xsl:when test="upper-case($chkName)=$chkName"><xsl:value-of select="$chkName"/></xsl:when>                                      
                                    <xsl:when test="not(upper-case($firstLetter)=$firstLetter)"><xsl:value-of select="replace(replace(concat(upper-case($firstLetter),substring-after($chkName,$firstLetter)),'_','-'),'\.','-')"/></xsl:when><!-- Make first letter upper case -->
                                    <xsl:when test="contains($chkName,'_') or(contains($chkName,'.'))"><xsl:value-of select="replace(replace($chkName,'_','-'),'\.','-')"/></xsl:when><!-- Adjust special characters -->
                                    <xsl:otherwise><xsl:value-of select="$chkName"/></xsl:otherwise>
                                </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="splitNameList">
                        <xsl:value-of select="camed:splitNewWords($newname,$newname,'true')"/>
                    </xsl:variable>
                    
                    <xsl:variable name="revisedName">
                        <xsl:for-each select="tokenize($splitNameList,'\|')">
                            <xsl:value-of select="if(position()=1) then camed:WordDefaults(.,$splitNameList[position() + 1],$mode) else concat('|',camed:WordDefaults(.,$splitNameList[position() + 1],$mode))"/>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="labelName"><xsl:value-of select="replace(replace($revisedName,'\|\|',' '),'\|',' ')"/></xsl:variable>
                    
                    <xsl:variable name="label"><xsl:value-of select="$labelName"/><xsl:value-of select="$nlchar"/></xsl:variable>
                    
                    <xsl:variable name="notes"><xsl:value-of select="camed:typesNotes(substring-before($thisType,'@'),substring-after($thisType,'@'),$cardinality)"/></xsl:variable>
                    
                    <xsl:variable name="rules">
                        <xsl:choose>
                            <xsl:when test="count(as:Rule/as:constraint/@condition) &gt; 0">
                            <xsl:for-each select="as:Rule/as:constraint[exists(@condition)]">
                                <xsl:if test="exists(@condition)">[condition[<xsl:value-of select="@condition"/>]action[<xsl:value-of select="@action"/>]xpath[<xsl:value-of select="@xpath"/>]]
                                </xsl:if>
                            </xsl:for-each>
                            </xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                    </xsl:variable>
                    
                    <xsl:variable name="completeContextList">
                        <xsl:choose>
                            <xsl:when test="count($items) &gt; 0"><xsl:value-of select="if(contains($items//@context[1],$thisContext)) then $items//@context[1] else if(string-length($items//@context[1]) &gt; 1) then concat($items//@context[1],'&#xA; ',$thisContext) else $thisContext"></xsl:value-of></xsl:when>
                            <xsl:when test="$annotations='true'"><xsl:value-of select="$thisContext"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="$thisContext"/></xsl:otherwise>
                        </xsl:choose>
                        <xsl:for-each select="$itemContextValues/context">
                            <xsl:value-of select="$nlchar"/> <xsl:value-of select="@context"/>
                        </xsl:for-each>
                    </xsl:variable>
                    
                    <xsl:variable name="completeContext"><xsl:for-each select="distinct-values(tokenize($completeContextList,'&#xA;'))"><xsl:if test="not(.='')"><xsl:value-of select="concat(normalize-space(.),'&#xA;')"/></xsl:if></xsl:for-each></xsl:variable>
                    
                    <xsl:choose>
                        <xsl:when test="$mode='CAM'">
                            <xsl:if test="position() = 1">
                                <xsl:message>  - <xsl:value-of select="@name"/></xsl:message>
                            </xsl:if>
                            <!-- xsl:message>output <xsl:value-of select="@name"/> - [<xsl:value-of select="$completeContext"/>]</xsl:message-->
                            <xsl:choose>
                                <xsl:when test="count($items) &gt; 0" exclude-result-prefixes="#all">
                                    <Property item="{@name}" UID="{$UIDvalue}" CCTS="{$CCTS-type}" cardinality="{$cardinality}" type="{substring-before($thisType,'@')}" mask="{substring-after($thisType,'@')}" children="{$children}" group="{$modelGroup}" context="{$completeContext}" where="{if (string-length($items//@where) &gt; 0) then concat($items//@where,', ',$fromSource) else $fromSource}"  definition="{if($items//@definition='') then $definition else $items//@definition}" label="{$label}" notes="{$notes}" language="{concat($langCode,$nlchar)}" anno="{if (string-length($items//@anno)=0) then $anno else  $items//@anno}" rules="{$rules}"/>
                                </xsl:when>
                                <xsl:when test="$annotations='true'" exclude-result-prefixes="#all">
                                    <Property item="{@name}" UID="{$UIDvalue}" CCTS="{$CCTS-type}" cardinality="{$cardinality}" type="{substring-before($thisType,'@')}" mask="{substring-after($thisType,'@')}" children="{$children}" group="{$modelGroup}" context="{$completeContext}" where="{$fromSource}" definition="{if(string-length($items//@definition) &gt; 0) then $items//@definition else $definition}" label="{$label}" notes="{$notes}" language="{concat($langCode,$nlchar)}" anno="{$anno}" rules="{$rules}"/>
                                </xsl:when>
                                <xsl:otherwise exclude-result-prefixes="#all">
                                    <Property item="{@name}" UID="{$UIDvalue}" CCTS="{$CCTS-type}" cardinality="{$cardinality}" type="{substring-before($thisType,'@')}" mask="{substring-after($thisType,'@')}" children="{$children}" group="{$modelGroup}" context="{$completeContext}" where="{$fromSource}" definition="{if(string-length($items//@definition) &gt; 0) then $items//@definition else $definition}" label="{$label}" notes="{$notes}" language="{concat($langCode,$nlchar)}" anno="{$items//@anno}" rules="{$rules}"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:variable name="itemDef"><!-- changed when anno used in lieu of definition -->
                                <xsl:choose>
                                    <xsl:when test="count($items) &gt; 0"><xsl:value-of select="$items//@definition"/></xsl:when>
                                    <xsl:when test="$annotations='true'"><xsl:value-of select="if (string-length($items//@definition)=0) then $definition else $items//@definition"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="$items//@definition"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>     
                            <xsl:variable name="entryName">
                                <xsl:choose>
                                    <xsl:when test="concat('/',@name) = $thisContext"><xsl:value-of select="@name"/></xsl:when>
                                    <xsl:when test="concat('@',@name) = $thisContext"><xsl:value-of select="@name"/></xsl:when>
                                    <xsl:otherwise><xsl:for-each select="tokenize($thisContext,'/')"><xsl:if test="position() &gt; 2">.</xsl:if><xsl:value-of select="if(contains(.,':')) then substring-after(.,':') else if (contains(.,'@')) then substring-after(.,'@') else ."/></xsl:for-each></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <xsl:variable name="properties" select="tokenize($entryName,'\.')"/>
                            
                            <!-- xsl:variable name="modelGroup">
                                <xsl:choose>
                                    <xsl:when test="camed:annotation/camed:documentation[@type='substitutionGroup']"><xsl:value-of select="camed:annotation/camed:documentation[@type='substitutionGroup']/text()"/></xsl:when>
                                    <xsl:when test="parent::as:Element/camed:annotation/camed:documentation[@type='substitutionGroup']"><xsl:value-of select="parent::as:Element/camed:annotation/camed:documentation[@type='substitutionGroup']/text()"/></xsl:when>
                                    <xsl:otherwise/>
                                </xsl:choose> 
                            </xsl:variable -->
                            <!-- xsl:variable name="entryNameValue">
                                <xsl:choose>
                                    <xsl:when test="contains($entryName,'\.')"><xsl:value-of select="$entryName"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="if(contains($entryName,':')) then substring-after($entryName,':') else $entryName"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable -->
                            
                            <xsl:element name="CCTSitem" exclude-result-prefixes="#all">
                                <xsl:attribute name="uniqueID" select="$UIDvalue"/>                         
                                <xsl:attribute name="dictionary-entry-name" select="$entryName"/>
                                <xsl:attribute name="type" select="if(starts-with($thisType,'@')) then 'ACC' else 'BCC'"/>
                                <xsl:attribute name="definition" select="$itemDef"/>
                                <xsl:attribute name="status">proposed</xsl:attribute>
                                <xsl:attribute name="objectClass" select="$properties[1]"/>
                                <xsl:attribute name="property" select="$properties[2]"/>
                                <xsl:attribute name="modelGroup" select="$modelGroup"/>
                                <xsl:attribute name="representation" select="substring-before($thisType,'@')"/>
                            </xsl:element>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:if>        
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
        
        <xsl:apply-templates select="as:Attribute">
            <xsl:with-param name="parentPath" select="$fullContext"/>
            <xsl:with-param name="mode" select="$mode"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="as:Element">
            <xsl:with-param name="parentPath" select="$fullContext"/>
            <xsl:with-param name="mode" select="$mode"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="comment()">
            <xsl:with-param name="parentPath" select="$fullContext"/>
            <xsl:with-param name="mode" select="$mode"/>
        </xsl:apply-templates>
    </xsl:template>
    
    <xsl:template match="comment()">
        <xsl:param name="parentPath"/>
        <xsl:param name="mode"/>
        <xsl:comment><xsl:value-of select="."/></xsl:comment>
    </xsl:template>
    
    <xsl:template match="as:Attribute">
        <xsl:param name="parentPath"/>
        <xsl:param name="mode"/>
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="fullContext" select="concat($parentPath,'/@',@name)"/>
        <!-- xsl:if test="$debug='true'"><xsl:message> - context: <xsl:value-of select="$fullContext"/></xsl:message></xsl:if -->
        <xsl:variable name="itemCount" select="count(//as:Attribute[@name=$this])"/>
        <xsl:variable name="shortContext">
            <!-- get last three parts of path -->
            <xsl:variable name="lastThree"><xsl:for-each select="reverse(tokenize(@path,'/'))"><xsl:if test="position() &lt;4"><xsl:value-of select="."/>/</xsl:if></xsl:for-each></xsl:variable>
            <xsl:for-each select="reverse(tokenize($lastThree,'/'))"><xsl:if test="position() &gt; 1">/</xsl:if><xsl:value-of select="."/></xsl:for-each>
        </xsl:variable>
        <xsl:variable name="itemPath" select="//as:Attribute[@name=$this][ends-with(@path,$shortContext)][1]/@path"/>
      
        <xsl:variable name="pathList" select="reverse(tokenize($fullContext,'/'))"/>
        <xsl:variable name="thisContext">
            <xsl:choose>
                <xsl:when test="count($pathList) &lt; 4">@<xsl:value-of select="$pathList[1]"/></xsl:when>
                <xsl:when test="count($pathList) &lt; 5">/<xsl:value-of select="$pathList[2]"/>@<xsl:value-of select="$pathList[1]"/></xsl:when>
                <xsl:otherwise><xsl:if test="$pathList[3] &gt; ' '">/<xsl:value-of select="$pathList[3]"/></xsl:if>/<xsl:value-of select="$pathList[2]"/>@<xsl:value-of select="$pathList[1]"/></xsl:otherwise>
            </xsl:choose> 
        </xsl:variable>
        
        <xsl:variable name="item" select="$LookupProperties//Property[@item = $itemName][1]"/><!-- removed upper-case() -->
     
        <xsl:choose>
            <xsl:when test="./as:Rule/as:constraint[starts-with(@action,'exclude')]"/>
            <xsl:when test="$targetNS='*' or($targetNS='' and(not(contains(@name,':')))) or(starts-with(@name,concat($targetNS,':')))">
                <xsl:if test="$debug='true' and(not($itemCount=1 or($itemPath[1]=@path)))"><xsl:message> Skip - <xsl:value-of select="concat('count: ',$itemCount,'- Path: ',$itemPath[1],' != ',@path)"></xsl:value-of></xsl:message></xsl:if>
                <xsl:if test="$itemCount=1 or($itemPath[1]=@path)"><!-- Either item is unique, or first time item is found, then write out context for everywhere it appears -->
                    <xsl:if test="$debug='true'"><xsl:message> Path <xsl:value-of select="concat(' - ',$itemPath[1])"/> <xsl:if test="not($itemPath[1]=@path)"><xsl:value-of select="concat(' | ',@path)"/></xsl:if> <xsl:if test="as:Rule/as:constraint[contains(@action,'lookup(')]"> - LOOKUP </xsl:if></xsl:message></xsl:if>
                    <xsl:variable name="anno">
                        <xsl:choose>
                            <xsl:when test="camed:annotation/camed:documentation[@type='Definition']"><xsl:value-of select="camed:annotation/camed:documentation[@type='Definition']/text()"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="replace(as:Rule/as:constraint/as:annotation/as:documentation[@type='Definition']/text(),'\|','; ')"></xsl:value-of></xsl:otherwise>
                        </xsl:choose> 
                    </xsl:variable>   
                    <xsl:variable name="modelGroup">
                        <xsl:choose>
                            <xsl:when test="camed:annotation/camed:documentation[@type='substitutionGroup']"><xsl:value-of select="camed:annotation/camed:documentation[@type='substitutionGroup']/text()"/></xsl:when>
                            <xsl:when test="parent::as:Element/camed:annotation/camed:documentation[@type='substitutionGroup']"><xsl:value-of select="parent::as:Element/camed:annotation/camed:documentation[@type='substitutionGroup']/text()"/></xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose> 
                    </xsl:variable>
                    <xsl:variable name="thisUID">
                        <xsl:choose>
                            <xsl:when test="camed:annotation/camed:documentation[@type='UID']"><xsl:value-of select="camed:annotation/camed:documentation[@type='UID']/text()"/></xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose> 
                    </xsl:variable>
                    <xsl:variable name="thisMask">
                        <xsl:choose>
                            <xsl:when test="$item//@type and(string-length($item//@mask) &gt; 0)"><xsl:value-of select="$item//@mask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'lookup(')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setDateMask')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setNumberMask')]/@action"/></xsl:call-template></xsl:when>
                            <xsl:otherwise><xsl:call-template name="setMask"><xsl:with-param name="mask" select="$item//@mask"/><xsl:with-param name="action" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/></xsl:call-template></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="thisType">
                        <xsl:choose>
                            <xsl:when test="$item//@type"><xsl:value-of select="$item//@type"/>@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'datatype')]"><xsl:value-of select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'datatype')][1]/@action,'('),')')"/>@</xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]"><xsl:call-template name="setType"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="replace(replace($thisMask,' \|',','),'''','')"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'lookup')]">list@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setDateMask')]"><xsl:call-template name="setDate"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberMask')]"><xsl:call-template name="setNumber"><xsl:with-param name="mask" select="$thisMask"/></xsl:call-template>@<xsl:value-of select="$thisMask"/></xsl:when>
                            <xsl:otherwise>string@<xsl:value-of select="$thisMask"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="UIDvalue">
                        <xsl:choose>
                            <xsl:when test="string-length($thisUID) &gt; 0"><xsl:value-of select="$thisUID"/></xsl:when>
                            <xsl:when test="$item//@UID"><xsl:value-of select="$item//@UID"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="if(contains(@name,':')) then concat(substring-before(@name,':'),'-') else 'cam-'"/><xsl:value-of select="generate-id()"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="rules">
                        <xsl:choose>
                            <xsl:when test="count(as:Rule/as:constraint/@condition) &gt; 0">
                                <xsl:for-each select="as:Rule/as:constraint[exists(@condition)]">
                                    <xsl:if test="exists(@condition)">[condition[<xsl:value-of select="@condition"/>]action[<xsl:value-of select="@action"/>]xpath[<xsl:value-of select="@xpath"/>]]
                                    </xsl:if>
                                </xsl:for-each>
                              </xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                    </xsl:variable>
                    
                    <xsl:variable name="chkName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                    <xsl:variable name="firstLetter" select="substring($chkName,1,1)"/>
                    
                    <xsl:variable name="newname">
                        <xsl:choose>
                            <xsl:when test="camed:WordMatch($chkName,'NIEM')='true'"><xsl:value-of select="$chkName"/></xsl:when>
                            <xsl:when test="$firstLetter=$chkName and(upper-case($firstLetter)=$firstLetter)"/><!-- skip one char upper case name -->
                            <xsl:when test="upper-case($chkName)=$chkName"><xsl:value-of select="$chkName"/></xsl:when>                                      
                            <xsl:when test="not(upper-case($firstLetter)=$firstLetter)"><xsl:value-of select="replace(replace(concat(upper-case($firstLetter),substring-after($chkName,$firstLetter)),'_','-'),'\.','-')"/></xsl:when><!-- Make first letter upper case -->
                            <xsl:when test="contains($chkName,'_') or(contains($chkName,'.'))"><xsl:value-of select="replace(replace($chkName,'_','-'),'\.','-')"/></xsl:when><!-- Adjust special characters -->
                            <xsl:otherwise><xsl:value-of select="$chkName"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="splitNameList">
                        <xsl:value-of select="camed:splitNewWords($newname,$newname,'true')"/>
                    </xsl:variable>
                    
                    <xsl:variable name="revisedName">
                        <xsl:for-each select="tokenize($splitNameList,'\|')">
                            <xsl:value-of select="if(position()=1) then camed:WordDefaults(.,$splitNameList[position() + 1],$mode) else concat('|',camed:WordDefaults(.,$splitNameList[position() + 1],$mode))"/>
                        </xsl:for-each>
                    </xsl:variable>
                    
                    <xsl:variable name="cardinality">
                        <xsl:choose>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'makeMandatory')]">required</xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'makeOptional')]">optional</xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                    </xsl:variable>

                    <xsl:variable name="labelName"><xsl:value-of select="replace(replace($revisedName,'\|\|',' '),'\|',' ')"/></xsl:variable>                
                    
                    <xsl:variable name="label"><xsl:value-of select="$labelName"/><xsl:value-of select="$nlchar"/></xsl:variable>
                    
                    <xsl:variable name="notes"><xsl:value-of select="camed:typesNotes(substring-before($thisType,'@'),substring-after($thisType,'@'),$cardinality)"/></xsl:variable>
                    
                    <xsl:variable name="itemContextValues">
                        <xsl:for-each select="//as:Attribute[@name=$this]">
                            <xsl:choose>
                                <xsl:when test="position()=1"/><!-- ignore first item -->
                                <xsl:otherwise>
                                    <xsl:variable name="chkPathList" select="reverse(tokenize(@path,'/'))"/>
                                    <xsl:variable name="thisContextHere">
                                        <xsl:choose>
                                            <xsl:when test="count($chkPathList) &lt; 4">/<xsl:value-of select="$chkPathList[1]"/></xsl:when>
                                            <xsl:when test="count($chkPathList) &lt; 5">/<xsl:value-of select="$chkPathList[2]"/>/<xsl:value-of select="$chkPathList[1]"/></xsl:when>
                                            <xsl:otherwise><xsl:if test="$chkPathList[3] &gt; ' '">/<xsl:value-of select="$chkPathList[3]"/></xsl:if>/<xsl:value-of select="$chkPathList[2]"/>/<xsl:value-of select="$chkPathList[1]"/></xsl:otherwise>
                                        </xsl:choose> 
                                    </xsl:variable>
                                    <context name="{@name}" context="{$thisContextHere}" seqn="{position()}"/>
                                    <!-- xsl:message><xsl:value-of select="@name"/> - <xsl:value-of select="$thisContextHere"/> - <xsl:value-of select="position()"/></xsl:message -->
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:variable>
                    
                    <xsl:variable name="completeContextList">
                        <xsl:choose>
                            <xsl:when test="count($item) &gt; 0"><xsl:value-of select="if(contains($item//@context[1],$thisContext)) then $item//@context[1] else if(string-length($item//@context[1]) &gt; 1) then concat($item//@context[1],'&#xA; ',$thisContext) else $thisContext"></xsl:value-of></xsl:when>
                            <xsl:when test="$annotations='true'"><xsl:value-of select="$thisContext"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="$thisContext"/></xsl:otherwise>
                        </xsl:choose>
                        <xsl:for-each select="distinct-values($itemContextValues/context/@context)"><xsl:value-of select="$nlchar"/><xsl:value-of select="."/></xsl:for-each>
                    </xsl:variable>
                    
                    <xsl:variable name="completeContext"><xsl:for-each select="distinct-values(tokenize($completeContextList,'&#xA;'))"><xsl:if test="not(.='')"><xsl:value-of select="concat(normalize-space(.),'&#xA;')"/></xsl:if></xsl:for-each></xsl:variable>
                    
                    <!-- xsl:message> attr context: <xsl:value-of select="@name"/> - <xsl:value-of select="$completeContext"/></xsl:message -->
                    
                    <xsl:choose>
                    <xsl:when test="$mode='CAM'">
                        <xsl:choose>
                            <xsl:when test="count($item) &gt; 0" exclude-result-prefixes="#all">
                                <Property item="{@name}" UID="{$UIDvalue}" CCTS="ASBIE" cardinality="{$cardinality}" type="{substring-before($thisType,'@')}" mask="{substring-after($thisType,'@')}"  children="" group="{$modelGroup}" context="{replace($completeContext,'@@','/@')}" where="{if (string-length($item//@where) &gt; 0) then concat($item//@where,', ',$fromSource) else $fromSource}" definition="{$item//@definition}" label="{$label}" notes="{$notes}" language="{concat($langCode,$nlchar)}" anno="{if (string-length($item//@anno)=0 and not($anno=$item//@definition)) then $anno else $item//@anno}" rules="{$rules}"/>
                            </xsl:when>
                            <xsl:when test="$annotations='true'" exclude-result-prefixes="#all">
                                <Property item="{@name}" UID="{$UIDvalue}" CCTS="ASBIE" cardinality="{$cardinality}" type="{substring-before($thisType,'@')}" mask="{substring-after($thisType,'@')}"  children="" group="{$modelGroup}" context="{replace($completeContext,'@@','/@')}"  where="{$fromSource}" definition="{if (string-length($item//@definition)=0) then $anno else $item//@definition}" label="{$label}" notes="{$notes}" language="{concat($langCode,$nlchar)}" anno="{if (string-length($item//@definition)=0) then '' else $anno}" rules="{$rules}"/>
                            </xsl:when>
                            <xsl:otherwise exclude-result-prefixes="#all">
                                <Property item="{@name}" UID="{$UIDvalue}" CCTS="ASBIE" cardinality="{$cardinality}" type="{substring-before($thisType,'@')}" mask="{substring-after($thisType,'@')}"  children="" group="{$modelGroup}" context="{replace($completeContext,'@@','/@')}"  where="{$fromSource}" definition="{$item//@definition}" label="{$label}" notes="{$notes}" language="{concat($langCode,$nlchar)}" rules="{$rules}"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise>  
                            <xsl:variable name="itemDef">
                                <xsl:choose>
                                    <xsl:when test="count($item) &gt; 0"><xsl:value-of  select="if (string-length($item//@anno)=0 and not($anno=$item//@definition)) then $anno else $item//@anno"/></xsl:when>
                                    <xsl:when test="$annotations='true'"><xsl:value-of select="if (string-length($item//@definition)=0) then $anno else $item//@definition"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="$item//@definition"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>     
                            <xsl:variable name="entryName">
                            <xsl:choose>
                                <xsl:when test="concat('/',@name) = $thisContext"><xsl:value-of select="@name"/></xsl:when>
                                <xsl:when test="concat('@',@name) = $thisContext"><xsl:value-of select="@name"/></xsl:when>
                                <xsl:otherwise><xsl:for-each select="tokenize(replace($thisContext,'@','/'),'/')"><xsl:if test="position() &gt; 2">.</xsl:if><xsl:value-of select="if(contains(.,':')) then substring-after(.,':') else if (contains(.,'@')) then substring-after(.,'@') else ."/></xsl:for-each></xsl:otherwise>
                            </xsl:choose>
                            </xsl:variable>
                            <xsl:variable name="properties" select="tokenize($entryName,'\.')"/>
                            <!-- xsl:variable name="entryNameValue">
                                <xsl:choose>
                                    <xsl:when test="contains($entryName,'\.')"><xsl:value-of select="$entryName"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="if(contains($entryName,':')) then substring-after($entryName,':') else $entryName"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable -->
                            <xsl:element name="CCTSitem" exclude-result-prefixes="#all">
                                <xsl:attribute name="uniqueID" select="$UIDvalue"/>                         
                                <xsl:attribute name="dictionary-entry-name" select="$entryName"/>
                                <xsl:attribute name="type" select="'ASCC'"/>
                                <xsl:attribute name="definition" select="$itemDef"/>
                                <xsl:attribute name="status">proposed</xsl:attribute>
                                <xsl:attribute name="objectClass" select="$properties[1]"/>
                                <xsl:attribute name="property" select="$properties[2]"/>
                                <xsl:attribute name="modelGroup" select="$modelGroup"/>
                                <xsl:attribute name="representation" select="substring-before($thisType,'@')"/>
                            </xsl:element>
                        
                    </xsl:otherwise>
                    </xsl:choose>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise/><!-- skip -->
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="node()"></xsl:template>
    
    <xsl:template name="setMask">
        <xsl:param name="mask"/>
        <xsl:param name="action"/>
        <xsl:choose>
            <xsl:when test="contains($action[1],'lookup(')">
                <xsl:variable name="lookupName" select="substring-before(substring-after($action[1],'lookup('),')')"/>
                <xsl:variable name="valuesString"><xsl:for-each select="$lookupTables[@name = $lookupName]/look:item/@name"><xsl:if test="position() &gt; 1">,</xsl:if><xsl:value-of select="."/></xsl:for-each></xsl:variable>
                <xsl:value-of select="if(string-length($valuesString) &gt; 0) then $valuesString else 'NO CODELIST VALUES'"/>
                <xsl:if test="$debug='true'"><xsl:message>Lookup: <xsl:value-of select="$lookupName"/> [<xsl:value-of select="$valuesString"/>]</xsl:message></xsl:if>
            </xsl:when>
            <xsl:when test="string-length($mask) &gt; 0"><xsl:value-of select="$mask"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="substring-before(substring-after($action[1],'('),')')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="setDate">
        <xsl:param name="mask"/>
        <xsl:choose>
            <xsl:when test="starts-with($mask,'YYYY-MM-DD''T''')">datetime</xsl:when>
            <xsl:when test="starts-with($mask,'HH:MI')">time</xsl:when>
            <xsl:otherwise>date</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="setNumber">
        <xsl:param name="mask"/>
        <xsl:choose>
            <xsl:when test="contains($mask,'.###')">float</xsl:when>
            <xsl:when test="contains($mask,'.#')">decimal</xsl:when>
            <xsl:when test="contains($mask,'.')">decimal</xsl:when>
            <xsl:when test="$mask='######'">integer</xsl:when>
            <xsl:otherwise>integer</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="setType">
        <xsl:param name="mask"/>
        <xsl:choose>
            <xsl:when test="contains($mask,'Valid'' |''Invalid')">boolean</xsl:when>
            <xsl:when test="contains($mask,'True'' |''False')">boolean</xsl:when>
            <xsl:when test="contains($mask,'true''|''false')">boolean</xsl:when>
            <xsl:when test="contains($mask,'|')">list</xsl:when>
            <xsl:otherwise>string</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="mergeDetails">
        <xsl:param name="elementItems"/>
        <xsl:param name="mode"/>
        <xsl:message>
            - merging with <xsl:value-of select="count($LookupProperties/ITEMSlookup/Property)"/> items
        </xsl:message>   
        <xsl:variable name="MergeItems">
            <mergeItems>
            <xsl:for-each select="$LookupProperties/ITEMSlookup/*">
                <xsl:sort select="./@item"/>
                <mergeItem item="{@item}" uid="{@UID}" itemsKey="{concat(@item,'+',@context)}"/>
            </xsl:for-each>
            </mergeItems>
        </xsl:variable>
        <xsl:variable name="UniqueItems">
            <uniqueItems>
                <xsl:for-each select="distinct-values($MergeItems/mergeItems/mergeItem/@itemsKey)">
                    <xsl:variable name="lookupKey" select="."/>
                    <xsl:variable name="context" select="substring-after(.,'+')"/>
                    <!-- xsl:message> - <xsl:value-of select="."/></xsl:message -->
                    <uniqueItem item="{substring-before(.,'+')}" context="{$context}" contextLength="{string-length($context)}"/>
                </xsl:for-each>
            </uniqueItems>
        </xsl:variable>           
        
        <xsl:variable name="MergedItemsUniqueList">
            <uniqueItems>
                <xsl:for-each select="distinct-values($UniqueItems/uniqueItems/uniqueItem/@item)">
                <xsl:variable name="lookupName" select="."/>
                <!-- Now look for duplicate named items -->
                <xsl:variable name="sameNamedItems" select="$UniqueItems/uniqueItems/uniqueItem[@item = $lookupName]"/>
                <xsl:if test="$debug = 'true'"><xsl:message> - checking <xsl:value-of select="."/> context for <xsl:value-of select="count($sameNamedItems)"/> items</xsl:message></xsl:if>
                <xsl:variable name="compareContextResults">
                    <xsl:choose>
                        <xsl:when test="count($sameNamedItems) = 1"><xsl:copy-of select="$sameNamedItems"/></xsl:when>
                        <xsl:otherwise>
                            <xsl:for-each select="$sameNamedItems">
                                <!-- Compare context - if another item has this same context - discard this one, otherwise keep this one -->
                                <xsl:variable name="thisContext" select="@context"/>
                                <xsl:choose>
                                    <xsl:when test="count($UniqueItems/uniqueItems/uniqueItem[contains(@context,$thisContext)]) &lt; 2"><xsl:copy-of select="."/></xsl:when>
                                    <xsl:otherwise/>
                                </xsl:choose>
                            </xsl:for-each>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="count($compareContextResults/uniqueItem) = 0"><!-- They are all match context - so find the one with the longest context -->
                            <xsl:for-each select="$UniqueItems/uniqueItems/uniqueItem[@item = $lookupName]"><!-- This should never be executed, but here to ensure always return result -->
                                <xsl:sort select="@contextLength" order="descending"/>
                                <xsl:if test="position() = 1"><xsl:copy-of select="."/><xsl:message> - pick top context <xsl:value-of select="$lookupName"/></xsl:message></xsl:if><!-- just get the top one in the list -->
                            </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise><xsl:copy-of select="$compareContextResults"/></xsl:otherwise>
                </xsl:choose>
                <!-- uniqueItem item="{substring-before(.,'+')}" context="{substring-after(.,'+')}" itemKey="{.}"/ -->
            </xsl:for-each>
            </uniqueItems>
        </xsl:variable>    
          
        <xsl:message>
            - merge from <xsl:value-of select="count($MergedItemsUniqueList/uniqueItems/uniqueItem)"/> unique items
            <xsl:if test="$debug = 'true'"><xsl:message> - items: <xsl:value-of select="$UniqueItems/uniqueItems/uniqueItem/@item"/></xsl:message></xsl:if>
        </xsl:message>
        <!-- xsl:for-each select="$UniqueItems/uniqueItems/*">
            <xsl:variable name="matchItem" select="./@item"/>
            <xsl:variable name="matchUID" select="./@uid"/>
            <xsl:message> - <xsl:value-of select="$matchItem"/> - <xsl:value-of select="$matchUID"/></xsl:message>
            <xsl:message> - find - <xsl:value-of select="$LookupProperties/ITEMSlookup/Property[@item = $matchItem][@UID = $matchUID][1]/@UID"/></xsl:message>
        </xsl:for-each -->
        <xsl:for-each select="$MergedItemsUniqueList/uniqueItems/*">
            <xsl:variable name="matchItem" select="./@item"/>
            <xsl:variable name="matchContext" select="./@context"/>
            <xsl:variable name="getEntry"  select="$LookupProperties/ITEMSlookup/Property[@item = $matchItem][@context = $matchContext][1]"/>
            <xsl:if test="not($getEntry/@item = $matchItem)">
                <xsl:message> - merge error - <xsl:value-of select="$matchItem"/> missed for context - <xsl:value-of select="$matchContext"/></xsl:message>
            </xsl:if>
            <xsl:for-each select="$getEntry[1]"><!-- Find the matched one record for originals for this unique -->
                <xsl:if test="$debug='true'"><xsl:message> - find - <xsl:value-of select="@item"/> - <xsl:value-of select="substring-before($matchContext,'&#xA;')"/></xsl:message></xsl:if>
            <xsl:variable name="find" select="@item"/>
            <!-- xsl:if test="not($elementItems//item[@name=$find])"><xsl:message>no match[<xsl:value-of select="$find"/>]</xsl:message></xsl:if -->
            <!-- xsl:if test="$elementItems//item[@name=$find]"><xsl:message>skipped[<xsl:value-of select="$find"/>]</xsl:message></xsl:if -->
            <xsl:choose>
               <xsl:when test="$mode='CAM'">
                   <xsl:choose>
                       <xsl:when test="not($elementItems//item[@name=$find])">
                           <xsl:if test="$debug='true'"><xsl:message> - merge 1 - <xsl:value-of select="@item"/></xsl:message></xsl:if>                          
                           <xsl:copy-of select="."/>
                       </xsl:when>
                       <xsl:when test="$targetNS='*' and(contains($find,':'))">
                           <xsl:if test="$debug='true'"><xsl:message> - merge 2 - <xsl:value-of select="@item"/></xsl:message></xsl:if>
                           <xsl:copy-of select="."/>
                       </xsl:when>
                       <xsl:when test="not($targetNS='') and(not(starts-with($find,concat($targetNS,':'))))">
                           <xsl:if test="$debug='true'"><xsl:message> - merge 3 - <xsl:value-of select="@item"/></xsl:message></xsl:if>
                           <xsl:copy-of select="."/>
                       </xsl:when>
                       <xsl:when test="not($elementItems//item[@name=$find][ends-with(@context,$matchContext)])">
                           <xsl:if test="$debug='true'"><xsl:message> - merge 4 - <xsl:value-of select="@item"/> - <xsl:value-of select="substring-before($matchContext,'&#xA;')"/></xsl:message></xsl:if>
                           <!-- xsl:message> - elements - <xsl:value-of select="$elementItems//item[@name=$find]/concat(@name,' - ',@context)"/></xsl:message -->
                           <xsl:copy-of select="."/>
                       </xsl:when>
                       <xsl:otherwise><xsl:if test="$debug='true'"><xsl:message> - <xsl:value-of select="$find"/> - skipped - <xsl:value-of select="substring-before($matchContext,'&#xA;')"/></xsl:message></xsl:if></xsl:otherwise>
                   </xsl:choose>
               </xsl:when>
              <xsl:otherwise>
                  <xsl:variable name="contexts" select="tokenize(@context,'&#xA;')"/>
                  <xsl:variable name="itemName" select="@item"/>
                  <xsl:variable name="itemType" select="@type"/>
                  <xsl:variable name="itemUID" select="@UID"/>
                  <xsl:variable name="itemDef" select="@definition"/>
                  <xsl:variable name="modelGroup" select="@modelGroup"/>
                  
                  <xsl:for-each select="$contexts">
                      <xsl:variable name="itemPath" select="if(starts-with(.,' ')) then substring-after(.,' ') else ."/>
                     <xsl:variable name="entryName">
                         <xsl:choose>
                             <xsl:when test="concat('/',$itemName) = $itemPath"><xsl:value-of select="$itemName"/></xsl:when>
                             <xsl:when test="concat('@',$itemName) = $itemPath"><xsl:value-of select="$itemName"/></xsl:when>
                             <xsl:otherwise><xsl:for-each select="tokenize($itemPath,'/')"><xsl:if test="position() &gt; 2">.</xsl:if><xsl:value-of select="if(contains(.,':')) then substring-after(.,':') else if(starts-with(.,'@') and position() &lt; 2) then '' else if (starts-with(.,'@')) then substring-after(.,'@') else ."/></xsl:for-each></xsl:otherwise>
                         </xsl:choose>
                     </xsl:variable>
                     <xsl:variable name="properties" select="tokenize($entryName,'\.')"/>

                      <!-- xsl:variable name="entryNameValue">
                          <xsl:choose>
                              <xsl:when test="contains($entryName,'\.')"><xsl:value-of select="$entryName"/></xsl:when>
                              <xsl:otherwise><xsl:value-of select="if(contains($entryName,':')) then substring-after($entryName,':') else $entryName"/></xsl:otherwise>
                          </xsl:choose>
                      </xsl:variable --> 
                     <xsl:element name="CCTSitem">
                         <xsl:attribute name="uniqueID" select="$itemUID"/>                         
                         <xsl:attribute name="dictionary-entry-name" select="$entryName"/>
                         <xsl:attribute name="type" select="if (contains(.,'@')) then 'ASCC' else if (string-length($itemType) = 0) then 'ACC' else 'BCC'"/>
                         <xsl:attribute name="definition" select="$itemDef"/>
                         <xsl:attribute name="status">proposed</xsl:attribute>
                         <xsl:attribute name="objectClass" select="$properties[1]"/>
                         <xsl:attribute name="property" select="$properties[2]"/>
                         <xsl:attribute name="modelGroup" select="$modelGroup"/>
                         <xsl:attribute name="representation" select="$itemType"/>
                     </xsl:element>
                   </xsl:for-each>
            </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:function name="camed:typesNotes">
        <xsl:param name="type"/>
        <xsl:param name="mask"/>
        <xsl:param name="cardinality"/>
        <xsl:variable name="optional" select="if(contains($cardinality,'optional')) then 'true' else 'false'"/>
        <!-- deduce notes text from type, mask, cardinality -->
        <xsl:choose>
            <xsl:when test="$type=''"/><!-- ignore empty type -->
            <xsl:when test="$type='any'"/><!-- ignore undefined type -->
            <xsl:when test="$type='anyAttribute'"/><!-- ignore undefined type -->
            <xsl:when test="contains($type,'REF')"/><!-- ignore XML type -->
            <xsl:when test="contains($type,'Name')"/><!-- ignore XML type -->
            <xsl:when test="$type='string'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter some text here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='token'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a single line of simple text here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='NMTOKEN'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a single token value here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='NMTOKENS'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a one or more token values here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='language'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a valid language code value <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='list'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>choose a value from the list<xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='boolean'">Please pick if this applies or not?<xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='datetime'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a date and time <xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='date'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a date here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='time'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a time here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='decimal'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter an amount value <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='float'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a numeric value <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='double'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a number value <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='integer'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a whole number here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='unsignedInt'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a whole number here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='unsignedLong'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a whole number here <xsl:if test="not($mask='')">(<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='anyURI'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a web address (http) <xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:when test="$type='base64Binary'"><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter location of a file attachment <xsl:value-of select="$nlchar"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="if($optional='false') then 'Please ' else 'Optionally '"/>enter a valid <xsl:value-of select="$type"/> <xsl:if test="not($mask='')"> (<xsl:value-of select="$mask"/>)</xsl:if><xsl:value-of select="$nlchar"/></xsl:otherwise>           
        </xsl:choose>
    </xsl:function>
    
</xsl:stylesheet>
