<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:w2e="http://thisSheet"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:random="http://exslt.org/random" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    exclude-result-prefixes="xsd w2e random camed xsi">
    <!-- CAM kit utility program -->
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, May, 2008 -->
    <!--
        Purpose  : Parse CAM template internal format (CXF) and validate an XML instance.
        Creates report of any errors in-line as CAM annotations.
        Original : 06/15/2008
        Initials : drrw
        CAM ver    : 1.1 
        Description : Designed as a lightweight implementation of CAM in XSLT.
                             Exploits the CAM internal template format (CXF) generated by jCAM eclipse editor.
        Updated  - Inits - Change
        06/15/08   drrw   Initial release (no rule checks)
        06/15/08   drrw   Refine structure and attribute output handling
        06/15/08   drrw   Build rule path
        06/15/08   drrw   Add repeatable check logic
        06/15/08   drrw   Add Mask rule check
        06/15/08   drrw   Add restrictValues check
        06/15/08   drrw   Build path model and check mandatory
        06/26/08   drrw   Add matching namespace uri on element
        06/26/08   drrw   Expand type/value checking code
        06/26/08   drrw   Remove associated schema from output xml???
        06/27/08   drrw   Refine mandatory path checking
        07/01/08   drrw   Implement content checking rules - setLength()
        07/01/08   drrw   Support CAMERROR attributes and error reporting mechanism
        07/02/08   drrw   Add restrictValues + rules logic for attributes
        07/02/08   drrw   Fix ancestor path building logic for attributes
        07/02/08   drrw   Implement Repeatable() item logic
        07/03/08   drrw   Add setLimit support in repeatable
        07/03/08   drrw   Ensure choice() item present
        07/03/08   drrw   Check element / attribute name is correct and known
        07/03/08   drrw   Datatype content rule checks
        07/03/08   drrw   Matches() regexp content mask checks
        07/26/08   drrw   Complete date/time mask handling
        07/26/08   drrw   Suppress w2e namespace outputting
        07/26/08   drrw   Enhance to full error reporting of all errors
        07/30/08   drrw   Number and string mask logic
        07/??/08   drrw   conditional rule logic - evaluate for string length
        07/??/08   drrw   xpath conditionals - evaluate
        08/01/08   drrw   root element name rule match check
        08/??/08   drrw   float type validation
        08/??/08   drrw   date with Z timezone
    -->
    <xsl:param name="ruleNamePath">rules.cxf</xsl:param>
    <xsl:param name="resultNamePath">cam-results.xml</xsl:param>
    
    <xsl:output method="html" indent="yes" omit-xml-declaration="yes"/>
    <xsl:output name="resultXML" method="xml" indent="yes" omit-xml-declaration="yes" />
    
    <xsl:variable name="version">1.12</xsl:variable>
    <xsl:variable name="cxfRules" select="document($ruleNamePath)"/>
    <xsl:variable name="namespaces" select="$cxfRules//as:Namespaces"/>
    <xsl:variable name="choiceSeparator">contains\(name\(\),'</xsl:variable>
    
    <xsl:variable name="loadNamespaces">
        <xsl:element name="this">
            <xsl:for-each select="$namespaces/as:namespace">
                <xsl:namespace name="{@prefix}">
                    <xsl:value-of select="."/>
                </xsl:namespace>
            </xsl:for-each>
            <!-- xsl:namespace name="xsi">xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"</xsl:namespace-->
        </xsl:element>
    </xsl:variable>
    
    <xsl:variable name="inputNS">   
            <xsl:for-each select="$namespaces/as:namespace">
                <xsl:element name="nsItem">
                <xsl:attribute name='prefix' select="@prefix"/>
                    <xsl:value-of select="."/>
                </xsl:element>
            </xsl:for-each>
    </xsl:variable>
    
    <xsl:template match="/">
        <xsl:message>Processing starting, version <xsl:value-of select="$version"/></xsl:message>
        
        <xsl:if test="count($cxfRules//as:AssemblyStructure/as:Structure/as:Element)=0">
            <xsl:message terminate="yes">Rules file is not valid CAM internal format (.cxf)</xsl:message>
        </xsl:if>
        <xsl:variable name="structure" select="*"/>
        <xsl:variable name="rootElement"><xsl:value-of select="child::*/name()"></xsl:value-of></xsl:variable>
        <xsl:variable name="rulesRoot" select="$cxfRules/as:CAM/as:AssemblyStructure/as:Structure[@taxonomy='XML']/as:Element[1]/@name"/>
        <xsl:if test="not($rootElement=$rulesRoot)"> 
                    <xsl:message terminate="yes">Root element [<xsl:value-of select="$rootElement"/>] does not match rule file [<xsl:value-of select="$rulesRoot"/>]</xsl:message>
        </xsl:if>
        <xsl:message>XML structure: <xsl:value-of select="$rootElement"/></xsl:message>
        <html>
            <body>
                <h3>CAM Template XSLT Validator</h3>
                <h4>Version <xsl:value-of select="$version"/></h4>
                <p>
                    <b>Using rules <xsl:value-of select="$ruleNamePath"/></b>
                </p>
                <h4>CAM Template HEADER information:</h4>
                <table>
                    <tr><td>Description: </td> <td><xsl:value-of select="$cxfRules//as:Header/as:Description"/></td></tr>
                    <tr><td>Owner: </td><td><xsl:value-of select="$cxfRules//as:Header/as:Owner"/></td></tr>
                    <tr><td>Date: </td><td><xsl:value-of select="$cxfRules//as:Header/as:DateTime"/></td></tr>
                    <tr><td>Version: </td><td><xsl:value-of select="$cxfRules//as:Header/as:Version"/></td></tr>
                </table>
                <h4>Validation:</h4>
                <table>
                    <tr><td>Root: </td><td><xsl:value-of select="$cxfRules//as:AssemblyStructure/as:Structure/as:Element[1]/@name"/></td></tr>
                    <tr><td>Elements: </td><td><xsl:value-of select="count(descendant::node())"/></td></tr>
                </table>
                
                <xsl:variable name="docPaths">
                    <paths>
                        <xsl:copy-of select="$loadNamespaces//namespace::*"/>
                        <xsl:for-each select="/child::*/*">
                            <group>
                            <xsl:element name="path">
                                <xsl:attribute name="name" select="concat('/',$rootElement)"></xsl:attribute>
                            </xsl:element>
                            <xsl:apply-templates select="." mode="path"><xsl:with-param name="parent" select="$rootElement"/></xsl:apply-templates><!-- no ancestor -->
                            </group>
                        </xsl:for-each>
                    </paths>
                </xsl:variable>
                
                <xsl:variable name="items">
                    <xsl:apply-templates select="$cxfRules/as:CAM/as:AssemblyStructure/as:Structure/(as:Element|as:Attribute)" mode="mandatory"/>
                </xsl:variable>
                
                <xsl:variable name="mandatoryChk">
                    <xsl:for-each select="$items/(as:Element|as:Attribute)">
                        <xsl:variable name="chk" select="@path"/>
                        <xsl:variable name="attrParent" select="substring-before(@path,'\/@')"/>
                        <xsl:variable name="elemParent" select="substring-before(@path,concat('/',@name))"/>
                        <xsl:variable name="match" >
                            <xsl:choose>
                                <xsl:when test="not(contains($chk,'@'))">
                                    <xsl:choose>
                                        <xsl:when test="string-length($elemParent) &gt; 0">
                                            <xsl:choose>
                                                <xsl:when test="$docPaths//group/path[@name=$elemParent]">
                                                    <xsl:value-of select="$docPaths//group/path[@name=$chk]/@name"/>
                                                </xsl:when>
                                                <xsl:otherwise>no error</xsl:otherwise>
                                            </xsl:choose>
                                        </xsl:when>
                                        <xsl:otherwise><xsl:value-of select="$docPaths//group/path[@name=$chk]/@name"/></xsl:otherwise>
                                    </xsl:choose> 
                                </xsl:when>
                                <xsl:when test="$docPaths//group/path[@name=$attrParent]">
                                    <xsl:value-of select="$docPaths//group/path[@name=$chk]/@name"/>
                                </xsl:when>
                                <xsl:otherwise>no error</xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:if test="string-length($match[1]) = 0"><camerror code="901001" text="Missing mandatory segment: {$chk}"></camerror></xsl:if>
                    </xsl:for-each>
                </xsl:variable>
                
                <!-- xsl:variable name="choiceChk">
                    <xsl:variable name="items" select="$cxfRules//as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]"/>
                    choices[<xsl:value-of select="count($items)"></xsl:value-of>] - check at least 1 of each choice selected - if parent present
                </xsl:variable -->
                
                <xsl:variable name="XMLresults">
                    <xsl:element name="{$rootElement}">
                        <xsl:for-each select="$loadNamespaces//namespace::*">
                            <xsl:if test="not(name()='w2e')">
                                <xsl:namespace name="{name()}" select="."/>
                            </xsl:if>
                        </xsl:for-each>
                        <xsl:copy-of select="/*/attribute::*"></xsl:copy-of>
                        <xsl:call-template name="insertErrors">
                            <xsl:with-param name="errorDetails" select="$mandatoryChk"/>
                        </xsl:call-template>
                        <xsl:apply-templates select="/child::*/*"><xsl:with-param name="parent" select="$rootElement"/></xsl:apply-templates><!-- no ancestor -->
                    </xsl:element>
                </xsl:variable>
                
                <xsl:result-document exclude-result-prefixes="w2e as camed" format="resultXML" indent="yes" href="{$resultNamePath}">
                    <xsl:comment> Generated by CAM XSLT Validator (version <xsl:value-of select="$version"/>) 
                        
                        CAM Template HEADER information (cxf): 
                        
                        Description: <xsl:value-of select="$cxfRules//as:Header/as:Description"/> 
                        Owner: <xsl:value-of select="$cxfRules//as:Header/as:Owner"/> 
                        Date: <xsl:value-of select="$cxfRules//as:Header/as:DateTime"/> 
                        Version: <xsl:value-of select="$cxfRules//as:Header/as:Version"/> 
                       <xsl:value-of select="'&#xD; &#xD;' "/>
                    </xsl:comment>
                    <xsl:copy-of select="$XMLresults"></xsl:copy-of>
                  
                </xsl:result-document>
                <xsl:variable name="countErrors">
                    <xsl:for-each select="$XMLresults//descendant-or-self::*/@*/name()">
                        <xsl:if test="contains(.,'CAMERRORCODE')"><error/></xsl:if>
                    </xsl:for-each>
                </xsl:variable>
                <ul>
                    <li> <xsl:value-of select="count($countErrors//error)"/> error items found. </li>
                </ul>
               
            </body>
        </html>

        <xsl:message>Finished</xsl:message>
    </xsl:template>
    
    <xsl:template match="node()">
        <xsl:param name="parent"></xsl:param>
        <xsl:param name="ancestor"></xsl:param>
       
        <xsl:variable name="rulePath"><xsl:value-of select="$ancestor"></xsl:value-of>/<xsl:value-of select="if(ends-with($ancestor,$parent)) then parent::*/name() else $parent"></xsl:value-of>/<xsl:value-of select="name()"></xsl:value-of></xsl:variable>
        <xsl:variable name="rules" select="$cxfRules//as:Element[@path=$rulePath]/as:Rule"/>
        <xsl:variable name="MaskRules" select="$rules//as:constraint[contains(@action,'Mask(')]"></xsl:variable>
        
        <!-- Repeat check -->
        <xsl:variable name="this" select="name()"/>
        <xsl:variable name="nodeNames" select="child::*/name()"/>
        <xsl:variable name="names">
            <xsl:for-each select="$nodeNames">
                <node name="{.}"/>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="namesList" select="distinct-values($nodeNames)"/>
        
        <xsl:variable name="ruleNames" select="$cxfRules//as:Element[@path=$rulePath]/as:Element/@name"></xsl:variable>
        
        <!-- Choice check -->
        <xsl:variable name="choiceChks">
            <xsl:for-each select="$ruleNames">
                <xsl:variable name="findRule" select="concat($rulePath,'/',.)"/>
                <xsl:variable name="temp" select="$cxfRules//as:Element[@path=$findRule]/as:Rule/as:constraint[contains(@action,'setChoice(')]"/>
                <xsl:copy-of select="$temp"/>
            </xsl:for-each>
        </xsl:variable>
      
        <xsl:variable name="chkForErrors">  
            
            <xsl:if test="count($rules)=0">
                <camerror code="101000" text="Error: unknown element, found [{$this}]."></camerror>
            </xsl:if>
            
            <xsl:for-each select="$namesList">
                <xsl:variable name="name" select="."/>
                <xsl:variable name="occurs" select="count($names/node[@name=$name])"/>
                <xsl:if test="$occurs &gt; 1">
                    <xsl:call-template name="chkRepeatRule"><xsl:with-param name="chkPath" select="concat($rulePath,'/',.)"/><xsl:with-param name="item" select="."/><xsl:with-param name="count" select="$occurs"/></xsl:call-template>
                </xsl:if>
            </xsl:for-each>
            
            <xsl:if test="count($choiceChks//as:constraint) &gt; 0">
            <xsl:for-each select="$choiceChks">
              
                <xsl:variable name="temp" select="substring-before(substring-after(as:constraint[1]/@action,'['),')]')"/>
                <xsl:variable name="templist" select="tokenize($temp,$choiceSeparator)"/>
                <xsl:variable name="chkChoices">
              
                <xsl:for-each select="$templist">
                    <xsl:variable name="item" select="substring-before(.,'''')"/>
                    <xsl:if test="count($names/node[@name=$item]) &gt; 0">found</xsl:if>
                </xsl:for-each>
                </xsl:variable>
                <xsl:if test="string-length($chkChoices) = 0">
                    <camerror code="202001" text="Error: missing choice element, should be one of [{$templist}]."></camerror>
                </xsl:if>
            </xsl:for-each>
            </xsl:if>
            
            <xsl:call-template name="chkRules">
                <xsl:with-param name="rules" select="$rules"/>
                <xsl:with-param name="maskRules" select="$MaskRules"/>
                <xsl:with-param name="value" select="text()"/>
            </xsl:call-template>    
             
        </xsl:variable>
        
        <xsl:variable name="thisNSuri"><xsl:call-template name="lookupNSuri"><xsl:with-param name="ns" select="substring-before(name(),':')"></xsl:with-param></xsl:call-template></xsl:variable>
        
        <xsl:element name="{name()}" namespace="{$thisNSuri}">
            <xsl:call-template name="insertErrors">
                <xsl:with-param name="errorDetails" select="$chkForErrors"/>
            </xsl:call-template> 
            <xsl:apply-templates select="attribute::*">
                <xsl:with-param name="ancestor" >
                    <xsl:choose>
                        <xsl:when test="ends-with($ancestor,concat('/',$parent))"><xsl:value-of select="$ancestor"/>/<xsl:value-of select="parent::*/name()"/></xsl:when>
                        <xsl:otherwise><xsl:value-of select="concat($ancestor,'/',$parent)"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:with-param>
                <xsl:with-param name="parent" select="name()"/>
            </xsl:apply-templates>
            <xsl:if test="not(descendant::*)"><xsl:value-of select="text()"/></xsl:if>
            <xsl:apply-templates select="*"><xsl:with-param name="ancestor" select="if(ends-with($ancestor,$parent)) then concat($ancestor,'/',parent::*/name()) else concat($ancestor,'/',$parent)"/><xsl:with-param name="parent"><xsl:value-of select="if(ends-with($ancestor,$parent)) then name() else parent::*/name()"/></xsl:with-param></xsl:apply-templates>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="node()" mode="path">
        <xsl:param name="parent"></xsl:param>
        <xsl:param name="ancestor"></xsl:param>
        <xsl:variable name="ns" select="node()/namespace-uri(.)"></xsl:variable>
        <xsl:variable name="rulePath"><xsl:value-of select="$ancestor"></xsl:value-of>/<xsl:value-of select="if(ends-with($ancestor,$parent)) then parent::*/name() else $parent"></xsl:value-of>/<xsl:value-of select="name()"></xsl:value-of></xsl:variable>
        <xsl:element name="path">
            <xsl:attribute name="name" select="$rulePath"></xsl:attribute>
        </xsl:element>
            <xsl:apply-templates select="attribute::*" mode="path"><xsl:with-param name="ancestor" select="concat($ancestor,'/',$parent)"/><xsl:with-param name="parent" select="name()"/></xsl:apply-templates>
            <xsl:apply-templates select="*" mode="path"><xsl:with-param name="ancestor" select="if(ends-with($ancestor,$parent)) then concat($ancestor,'/',parent::*/name()) else concat($ancestor,'/',$parent)"/><xsl:with-param name="parent"><xsl:value-of select="if(ends-with($ancestor,$parent)) then name() else parent::*/name()"/></xsl:with-param></xsl:apply-templates>
     </xsl:template>   
    
    <xsl:template  match="attribute::node()">
        <xsl:param name="parent"></xsl:param>
        <xsl:param name="ancestor"></xsl:param>
        <xsl:variable name="rulePath">
            <xsl:choose>
                <xsl:when test="ends-with($ancestor,concat('/',$parent))"><xsl:value-of select="$ancestor"></xsl:value-of>/@<xsl:value-of select="name()"></xsl:value-of></xsl:when>
                <xsl:otherwise><xsl:value-of select="$ancestor"></xsl:value-of>/<xsl:value-of select="$parent"></xsl:value-of>/@<xsl:value-of select="name()"></xsl:value-of></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="rules" select="$cxfRules//as:Attribute[@path=$rulePath]/as:Rule"/>
        <xsl:variable name="MaskRules" select="$rules//as:constraint[contains(@action,'Mask(')]"></xsl:variable>
       
        <xsl:variable name="chkForErrors">
            <xsl:if test="count($rules)=0">
                <camerror code="101001" text="Error: unknown attribute, found [{name()}]."></camerror>
            </xsl:if>
            <xsl:call-template name="chkRules">
                <xsl:with-param name="rules" select="$rulePath"/>
                <xsl:with-param name="maskRules" select="$MaskRules"/>
                <xsl:with-param name="value" select="."/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:call-template name="insertErrors">
            <xsl:with-param name="errorDetails" select="$chkForErrors"/>
            <xsl:with-param name="name" select="name()"/>
        </xsl:call-template> 
        
        <xsl:copy-of select="."/>
    
    </xsl:template>
    
    <xsl:template  match="attribute::node()" mode="path">
        <xsl:param name="parent"></xsl:param>
        <xsl:param name="ancestor"></xsl:param>
        <xsl:variable name="rulePath"><xsl:value-of select="$ancestor"></xsl:value-of>/<xsl:value-of select="$parent"></xsl:value-of>/@<xsl:value-of select="name()"></xsl:value-of></xsl:variable>
        <xsl:element name="path">
            <xsl:attribute name="name" select="$rulePath"></xsl:attribute>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="as:Element" mode="mandatory">
        <xsl:if test="as:Rule/as:constraint[@action='makeMandatory()']">
            <xsl:element name="as:Element">
                <xsl:attribute name="name" select="@name"/>
                <xsl:attribute name="path" select="@path"/>
                <xsl:copy-of select="as:Rule"/>
            </xsl:element>
        </xsl:if>
        <xsl:apply-templates select="as:Attribute" mode="mandatory"/>
        <xsl:apply-templates select="as:Element" mode="mandatory"/>
    </xsl:template>
    
    <xsl:template match="as:Attribute" mode="mandatory">
        <xsl:if test="as:Rule/as:constraint[@action='makeMandatory()']">
            <xsl:element name="as:Attribute">
                <xsl:attribute name="path" select="@path"/>
                <xsl:copy-of select="as:Rule"/>
            </xsl:element>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="text()" mode="mandatory"></xsl:template>
    
    <xsl:template name="insertErrors">
        <xsl:param name="errorDetails"/>
        <xsl:param name="name"/>
        <xsl:variable name="errorCodes">
            <xsl:for-each select="$errorDetails/camerror"><xsl:value-of select="@code"/>;</xsl:for-each>
        </xsl:variable>
        <xsl:variable name="errorMsgs">
            <xsl:for-each select="$errorDetails/camerror"><xsl:value-of select="@text"/>;</xsl:for-each>
        </xsl:variable>
        <xsl:if test="string-length($errorCodes) &gt; 0">
            <xsl:attribute name="{concat('CAMERROR',$name)}" select="$errorMsgs"/>
            <xsl:attribute name="{concat('CAMERRORCODE',$name)}" select="$errorCodes"/>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="chkRules">
        <xsl:param name="rules"/>
        <xsl:param name="maskRules"/>
        <xsl:param name="value"/>
        
        <xsl:if test="$maskRules">
            <xsl:call-template name="chkMaskRules">
                <xsl:with-param name="chkRules" select="$maskRules"/>
                <xsl:with-param name="value" select="$value"/>
            </xsl:call-template>
        </xsl:if>
       
        <xsl:if test="$rules//as:constraint[contains(@action,'restrictValues(')]">
            <xsl:variable name="chkValuesStr" select="$rules//as:constraint[contains(@action,'restrictValues(')]/substring-after(@action,'restrictValues(')"></xsl:variable>
            <xsl:variable name="chkValues" select="tokenize($chkValuesStr,'\|')"></xsl:variable>
            <xsl:variable name="found">
                <xsl:for-each select="$chkValues">
                    <xsl:if test="$value=substring-before(substring-after(.,''''),'''')">true</xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:if test="not($found='true')"><camerror code="2001001" text="invalid value [{$value}], should be one of [{substring-before($chkValuesStr,')')}]."></camerror></xsl:if>
        </xsl:if>
        
        <xsl:if test="$rules//as:constraint[contains(@action,'setLength(')]">
            <xsl:variable name="Action" select="$rules//as:constraint[contains(@action,'setLength(')]/substring-after(@action,'(')"/>
            <xsl:variable name="minLen" select="if(contains($Action,'-')) then substring-before($Action,'-') else '0'"/>
            <xsl:variable name="maxLen" select="if(contains($Action,'-')) then substring-before(substring-after($Action,'-'),')') else substring-before($Action,')')"></xsl:variable>
            <xsl:variable name="strLen" select="string-length($value)"/>
            
            <xsl:if test="$minLen &gt; '0'">
                <xsl:if test="$strLen &lt; xsd:integer($minLen)">
                    <camerror code="301001" text="[{$value}] shorter than minimum length {$minLen}."/>
                </xsl:if>
            </xsl:if>
            <xsl:if test="$maxLen &gt; '0'">
                <xsl:if test="$strLen &gt; xsd:integer($maxLen)">
                    <camerror code="301002" text="[{$value}] longer than maximum length {$maxLen}."/>
                </xsl:if>
            </xsl:if>
        </xsl:if>
        
        <!-- 
        <xsl:when test="$chkAction='setNumberRange'"></xsl:when>
        <xsl:when test="$chkAction='setValue'"></xsl:when>
        <xsl:when test="$chkAction='setDefault'"></xsl:when>
        <xsl:when test="$chkAction='setValue'"></xsl:when>
        <xsl:when test="$chkAction='datatype'"></xsl:when>
        -->
        
    </xsl:template>

    <xsl:template name="chkMaskRules">
        <xsl:param name="chkRules"/>
        <xsl:param name="value"/>
        
        <xsl:for-each select="$chkRules">
            
            <!-- xsl:choose>
                <xsl:when test="@condition"></xsl:when>
                <xsl:otherwise -->

                    <xsl:variable name="chkAction" select="substring-before(./@action,'(')"/>
                    <xsl:variable name="Action" select="substring-after(./@action,'(')"/>

                    <xsl:choose>
                        <xsl:when test="$chkAction='setDateMask'"> 
                            <!-- tokenize value by delimiter -->
                            <xsl:variable name="date" select="if(contains($Action,'\\')) then tokenize($value,'\\') else if(contains($Action,'-')) then tokenize($value,'-') else $value"/>
                            <xsl:variable name="dateRules" select="if(contains($Action,'\\')) then tokenize($Action,'\\') else if(contains($Action,'-')) then tokenize($Action,'-') else $Action"/>
                   
                            <xsl:for-each select="$date">
                                <xsl:variable name="chk" select="."/>
                                <xsl:variable name="this" select="position()"/>
                                <xsl:variable name="rule" select="$dateRules[$this]"/>
                                <xsl:choose>
                                    <xsl:when test="$rule='YYYY'">
                                        <xsl:if test="not(matches($chk,'[0-9]+')) or(matches($chk,'\p{L}+')) or(not(string-length($chk)=4))"><camerror code="401000" text="Error: invalid year, found [{$chk}] in [{$value}]."></camerror></xsl:if>
                                    </xsl:when>
                                    <xsl:when test="$rule='MM'">
                                        <xsl:if test="not(matches($chk,'[0-9]+')) or(matches($chk,'\p{L}+')) or(not(string-length($chk)=2)) or($chk &gt; '12') or($chk &lt; '01')"><camerror code="401001" text="Error: invalid month, found [{$chk}] in [{$value}]."></camerror></xsl:if>
                                    </xsl:when>
                                    <xsl:when test="$rule='DD'">
                                        <xsl:variable name="ddValue">
                                        <xsl:choose>
                                            <xsl:when test="string-length($chk) &lt; 3"><xsl:value-of select="$chk"/></xsl:when>
                                            <xsl:when test="contains($chk,'T')"><xsl:value-of select="substring-before($chk,'T')"/></xsl:when>
                                            <xsl:otherwise><xsl:value-of select="$chk"/></xsl:otherwise>
                                        </xsl:choose>
                                        </xsl:variable>
                                        <xsl:if test="not(matches($ddValue,'[0-9]+')) or(matches($ddValue,'\p{L}+')) or(not(string-length($ddValue)=2)) or($ddValue &gt; '31') or($ddValue &lt; '01')"><camerror code="401003" text="Error: invalid day, found [{$ddValue}] in [{$value}]."></camerror></xsl:if>
                                    </xsl:when>
                                    <xsl:when test="starts-with($rule,'DD')">
                                        <xsl:variable name="ddValue">
                                            <xsl:choose>
                                                <xsl:when test="string-length($chk) &lt; 3"><xsl:value-of select="$chk"/></xsl:when>
                                                <xsl:when test="contains($chk,'T')"><xsl:value-of select="substring-before($chk,'T')"/></xsl:when>
                                                <xsl:otherwise><xsl:value-of select="$chk"/></xsl:otherwise>
                                            </xsl:choose>
                                        </xsl:variable>
                                        <xsl:if test="not(matches($ddValue,'[0-9]+')) or(matches($ddValue,'\p{L}+')) or(not(string-length($ddValue)=2)) or($ddValue &gt; '31') or($ddValue &lt; '01')"><camerror code="401004" text="Error: invalid day, found [{$ddValue}] in [{$value}]."></camerror></xsl:if>
                                        <xsl:if test="contains($rule,'T')">
                                            <xsl:variable name="tRules" select="tokenize(substring-after($rule,'''T'''),':')"/>
                                            <xsl:variable name="tValues" select="tokenize(substring-after($value,'T'),':')"/>
                                            <xsl:for-each select="$tValues">
                                                <xsl:variable name="this" select="position()"/>
                                                <xsl:call-template name="chkTime">
                                                    <xsl:with-param name="rule" select="$tRules[$this]"/>
                                                    <xsl:with-param name="value" select="."/>
                                                    <xsl:with-param name="content" select="$chk"/>
                                                </xsl:call-template>
                                            </xsl:for-each>
                                        </xsl:if>
                                    </xsl:when>
                                    <xsl:when test="$rule ='HH' or($rule='MI') or($rule='SS')">
                                        <xsl:call-template name="chkTime">
                                            <xsl:with-param name="rule" select="$rule"/>
                                            <xsl:with-param name="value" select="."/>
                                            <xsl:with-param name="content" select="$chk"/>
                                        </xsl:call-template>
                                    </xsl:when>
                                    <xsl:when test="contains($rule,'HH')">
                                        <xsl:variable name="tRules" select="tokenize($rule,':')"/>
                                        <xsl:variable name="tValues" select="tokenize($value,':')"/>
                                        <xsl:for-each select="$tRules">
                                            <xsl:variable name="this" select="position()"/>
                                            <xsl:call-template name="chkTime">
                                                <xsl:with-param name="rule" select="."/>
                                                <xsl:with-param name="value" select="$tValues[$this]"/>
                                                <xsl:with-param name="content" select="$chk"/>
                                            </xsl:call-template>
                                        </xsl:for-each>
                                    </xsl:when>
                                    <xsl:when test="string-length($rule)=0"></xsl:when>
                                    <xsl:otherwise><!-- xsl:message> [<xsl:value-of select="."/>][<xsl:value-of select="$rule"/>]</xsl:message --></xsl:otherwise>
                                </xsl:choose>
                            </xsl:for-each>
                            <!-- xsl:message>data value  D - <xsl:value-of select="matches($value,'9*')"/>/<xsl:value-of select="$Action"/> - <xsl:value-of select="$value"/></xsl:message -->
                        </xsl:when>
                        <xsl:when test="$chkAction='setNumberMask'">
                            <!-- xsl:variable name="digits">
                                <xsl:choose>
                                    <xsl:when test="contains($Action,'.')"><xsl:value-of select="substring-before($Action,'.')"></xsl:value-of></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="$Action"/></xsl:otherwise>
                                </xsl:choose>
                                </xsl:variable -->
                            <!-- check for shorthand notation #8.## etc -->
                            <xsl:variable name="decimal" select="substring-after($Action,'.')"/>
                            <!-- check is number mask contains non-numerics / minus, plus, paren's, $, etc -->
                            <xsl:choose>
                                <xsl:when test="string-length($decimal) &gt; 0">
                                    <xsl:if test="not(contains($value,'.'))"><camerror code="405000" text="Error: invalid decimal number, missing decimal point in [{$value}]."></camerror></xsl:if>
                                    <xsl:variable name="digits" select="replace($value,'\.','')"></xsl:variable>
                                    <xsl:variable name="decimals" select="substring-after($value,'.')"/>
                                    <xsl:if test="string-length($decimals) &lt; string-length(substring-after($value,'.'))"><camerror code="405004" text="Error: invalid decimal number, found too many decimal places in [{$value}] but expecting [{$Action}]."></camerror></xsl:if>
                                    <xsl:if test="not(matches($digits,'[0-9]+')) or(matches($digits,'\p{L}+')) or(string-length($Action) &lt; string-length($value))"><camerror code="405001" text="Error: invalid decimal number, found [{$value}] but expecting [{$Action}]."></camerror></xsl:if>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:if test="contains($value,'.')"><camerror code="405002" text="Error: invalid number, contains decimal point in [{$value}]."></camerror></xsl:if>
                                    <xsl:if test="not(matches($value,'[0-9]+')) or(matches($value,'\p{L}+')) or(string-length($Action) &lt; string-length($value))"><camerror code="405003" text="Error: invalid number, found [{$value}] but expecting [{$Action}]."></camerror></xsl:if>
                                </xsl:otherwise>
                            </xsl:choose>                            
  
                        </xsl:when>
  
                        <xsl:when test="$chkAction='setStringMask'"></xsl:when>
                        <xsl:otherwise>
                            <!-- xsl:message>data value - <xsl:value-of select="data($value)"/></xsl:message -->
                        </xsl:otherwise>
                    </xsl:choose>                    
                <!-- /xsl:otherwise>
            </xsl:choose -->
            
        </xsl:for-each>
        
    </xsl:template>
    
    <xsl:template name="chkRepeatRule">
        <xsl:param name="chkPath"/>
        <xsl:param name="item"/>
        <xsl:param name="count"/>
        <xsl:variable name="rules" select="$cxfRules//as:Element[@path=$chkPath]/as:Rule"/>
        <xsl:choose>
            <xsl:when test="$rules//as:constraint[@action='makeRepeatable()']">
                <xsl:if test="$rules//as:constraint[contains(@action,'setLimit(')]">
                    <xsl:variable name="limit" select="substring-before(substring-after($rules//as:constraint[contains(@action,'setLimit(')]/@action,'('),')')"></xsl:variable>
                    <xsl:if test="$count &gt; xsd:integer($limit)">
                        <camerror code="701002" text="Error [{$item}] occurs {$count} times (limit is for {$limit} only)"/>
                    </xsl:if>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise><camerror code="701001" text="Error [{$item}] occurs more than once"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="lookupNSuri">
        <xsl:param name="ns"/>
        <xsl:variable name="temp" select="$inputNS/nsItem[@prefix=$ns]/."/>
        <xsl:value-of select="$temp"/>
    </xsl:template>
    
    <xsl:template name="chkTime">
        <xsl:param name="rule"/>
        <xsl:param name="value"/>
        <xsl:param name="content"/>
        <xsl:choose>
            <xsl:when test="$rule='HH'">
                <xsl:if test="not(matches($value,'[0-9]+')) or(matches($value,'\p{L}+')) or(not(string-length($value)=2)) or($value &gt; '23') or($value &lt; '00')"><camerror code="401005" text="Error: invalid hour, found [{$value}] in [{$content}]."></camerror></xsl:if>
            </xsl:when>
            <xsl:when test="$rule='MI'">
                <xsl:if test="not(matches($value,'[0-9]+')) or(matches($value,'\p{L}+')) or(not(string-length($value)=2)) or($value &gt; '59') or($value &lt; '00')"><camerror code="401006" text="Error: invalid minute, found [{$value}] in [{$content}]."></camerror></xsl:if>
            </xsl:when>
            <xsl:when test="$rule='SS'">
                <xsl:if test="not(matches($value,'[0-9]+')) or(matches($value,'\p{L}+')) or(not(string-length($value)=2)) or($value &gt; '59') or($value &lt; '00')"><camerror code="401007" text="Error: invalid seconds, found [{$value}] in [{$content}]."></camerror></xsl:if>
            </xsl:when>
            <xsl:when test="starts-with($rule,'SS')">
                <xsl:variable name="seconds" select="substring($value,1,2)"/>
                <xsl:if test="not(matches($seconds,'[0-9]+')) or(matches($seconds,'\p{L}+')) or(not(string-length($seconds)=2)) or($seconds &gt; '59') or($seconds &lt; '00')"><camerror code="401007" text="Error: invalid second, found [{$seconds}] in [{$content}]."></camerror></xsl:if>
                <xsl:if test="contains($rule,'Z')">
                    <xsl:variable name="zones" select="substring-after($value,'-')"/>
                    <xsl:if test="not(matches($zones,'[0-9]+')) or(matches($zones,'\p{L}+')) or(not(string-length($zones)=2)) or($zones &gt; '15') or($zones &lt; '00')"><camerror code="401008" text="Error: invalid time zone, found [{$zones}] in [{$value}]."></camerror></xsl:if>
                </xsl:if>
                <xsl:if test="contains($rule,'.S')">
                    <xsl:variable name="decimal" select="substring-after($value,'.')"/>
                    <xsl:variable name="splitSeconds">
                        <xsl:choose>
                            <xsl:when test="contains($decimal,'-')"><xsl:value-of select="substring-before($decimal,'-')"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="$decimal"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:if test="not(matches($splitSeconds,'[0-9]+')) or(matches($splitSeconds,'\p{L}+'))"><camerror code="401009" text="Error: invalid time split second, found [{$splitSeconds}] in [{$content}]."></camerror></xsl:if>
                </xsl:if>
            </xsl:when>
        </xsl:choose>
        
    </xsl:template>

</xsl:stylesheet>
